//#include <SPI.h>
#include <RH_RF95.h>
#include <Adafruit_GPS.h>

//Setup Serial Communication with GPS featherwing
#define GPSSerial Serial1
Adafruit_GPS GPS(&GPSSerial);

//for feather32u4
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 7

//ignition relay pins
#define closeIgnitionCircuit 11
#define openIgnitionCircuit 18

#define vBat A9

// Change to 434.0 or other frequency, must match RX's freq!
#define RF95_FREQ 915.0

// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

bool debug = false;
const int gpsSamples = 6000;

uint32_t timer = millis();

uint32_t sendNum = 0;
unsigned int armed = 0;
unsigned int ignition = 0;
unsigned int recData = 0;
String missionStatus = "Idle";  
uint32_t ignitionTime = millis();
uint32_t flightTime = millis();

//arrays to record GPS data
unsigned int gpsT[gpsSamples];
float altMSL[gpsSamples];
float gpsLat[gpsSamples];
int gpsLatDir[gpsSamples];
float gpsLon[gpsSamples];
int gpsLonDir[gpsSamples];
float altGPS[gpsSamples];

float cntDWN = 5.000;
uint32_t strtTime = millis();

void setup()
{
  Serial.begin(115200);
  delay(100);
  
  // 9600 NMEA is the default baud rate for Adafruit MTK GPS's- some use 4800
  GPS.begin(9600);
  delay(100);
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_GGAONLY);
  delay(100);
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_10HZ); // 10 Hz update rate
  delay(100);
  
  pinMode(RFM95_RST, OUTPUT);
  digitalWrite(RFM95_RST, HIGH);
  pinMode(closeIgnitionCircuit, OUTPUT);
  pinMode(openIgnitionCircuit, OUTPUT);
  digitalWrite(closeIgnitionCircuit, LOW);
  delay(5);
  digitalWrite(openIgnitionCircuit, HIGH);
  delay(10);
  digitalWrite(openIgnitionCircuit, LOW);

  // manual reset of radio module
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);

  //initialize radio module
  while (!rf95.init()) {
    delay(1000);
  }
  
  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  if (!rf95.setFrequency(RF95_FREQ)) {
    delay(1000);
  }
  
  //Set Radio Module Power Output
  rf95.setTxPower(23, false);

  //delay to ensure setup completes
  delay(1000);
}

int16_t packetnum = 0;

void loop()
{
  // read data from the GPS in the 'main loop'
  char c = GPS.read();

  //if there is new GPS data, parse it
  if (GPS.newNMEAreceived()) {
    //if we fail to parse a sentence just wait for another
    if (!GPS.parse(GPS.lastNMEA())) // this also sets the newNMEAreceived() flag to false
      return;
  }

  //set transmission rate
  if (millis() - timer > 200) {
    timer = millis(); // reset the timer

    //run down countdown timer if armed
    if (armed) {
      cntDWN = cntDWN - (millis() - strtTime) * 0.001;
      strtTime = millis();
      
      if (cntDWN <= 0) {
        missionStatus = "Ignition";
        ignition = 1;
        //flip ignition relay closed
        digitalWrite(openIgnitionCircuit, LOW);
        delay(5);
        digitalWrite(closeIgnitionCircuit, HIGH);
        delay(10);
        digitalWrite(closeIgnitionCircuit, LOW);

        //reset variables
        cntDWN = 5.000;
        armed = 0;

        //start timing ignition
        ignitionTime = millis();
      }
    }

    if (ignition) {
      //Write Relay pin low if 3 seconds have passed
      if (millis() - ignitionTime > 3000) {
        missionStatus = "In Flight";
        ignition = 0;
        //flip ignition relay open
        digitalWrite(openIgnitionCircuit, HIGH);
        delay(10);
        digitalWrite(openIgnitionCircuit, LOW);

        //start timing flight time
        flightTime = millis();
      }

    }

    //create a char array buffer
    char sendStr[100] = "";

    //build char array to transmit
    //increment transmission number
    itoa(sendNum++, sendStr, 10);
    strcat(sendStr, ",");
    //latitude
    dtostrf(GPS.latitude, 9, 4, sendStr + strlen(sendStr));
    sendStr[strlen(sendStr)] += GPS.lat;
    strcat(sendStr, ",");
    //longitude
    dtostrf(GPS.longitude, 9, 4, sendStr + strlen(sendStr));
    sendStr[strlen(sendStr)] += GPS.lon;
    strcat(sendStr, ",");
    //altitude
    dtostrf(GPS.altitude, 5, 1, sendStr + strlen(sendStr));
    strcat(sendStr, ",");
    dtostrf(cntDWN, 3, 3, sendStr + strlen(sendStr));
    strcat(sendStr, ",");

    if (missionStatus == "Idle" || missionStatus == "Ignition"){
      itoa(0, sendStr + strlen(sendStr), 10);
      strcat(sendStr, ",");
    }else{
      itoa(1, sendStr + strlen(sendStr), 10);
      strcat(sendStr, ",");
    }

    if (missionStatus == "Idle" || missionStatus == "Armed"){    
      itoa(0, sendStr + strlen(sendStr), 10);
      strcat(sendStr, ",");
    }else{
      itoa(1, sendStr + strlen(sendStr), 10);
      strcat(sendStr, ",");
    }
    
    dtostrf(2 * 3.3 * (analogRead(vBat) / 1024.0), 4, 2, sendStr + strlen(sendStr));
    strcat(sendStr, "!");

    //Transmit data packet
    rf95.send((uint8_t *)sendStr, strlen(sendStr));
    rf95.waitPacketSent();

    if (debug){
      Serial.println(sendStr);
    }

    // Now wait for a reply
    uint8_t buf[RH_RF95_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);

    if (rf95.waitAvailableTimeout(50))
    {
      // Should be a reply message for us now  
      if (rf95.recv(buf, &len))
      {
        String recvd = (char*)buf;
        //Serial.println("Got reply: ");
        //Serial.println(recvd);
        if (recvd == "LAUNCH!") {
          strtTime = millis();
          missionStatus = "Armed";
          armed = 1;
          recData = 1;
        }
        if (recvd == "ABORT!") {
          missionStatus = "Idle";
          armed = 0;
          ignition = 0;
          cntDWN = 5.000;
          recData = 0;
        }
        if (recvd == "EOM!") {
          missionStatus = "Idle";
          armed = 0;
          ignition = 0;
          cntDWN = 5.000;
          recData = 0;
        }
      }

    }

  }
}
